#include <18F4550.h>
#fuses XT         // Crystal osc <= 4mhz
#fuses NOWDT      // No Watch Dog Timer
#fuses NOPROTECT  // Code not protected from reading
#fuses NOLVP      // No low voltage programing, B3(PIC16) or B5(PIC18) used for I/O
#fuses NODEBUG    // No debug mode for use with ICD
#fuses USBDIV     // USB clock source comes from PLL divide by 2
#fuses CPUDIV1    // No System Clock Postscaler
#fuses PLL1       // No PLL PreScaler
#fuses VREGEN     // USB voltage regulator enabled
#use standard_io(D)
#use delay(clock=4000000)
#use rs232(baud=1200, parity=E, bits=7, xmit=PIN_C6, rcv=PIN_C7, TIMEOUT=250, STREAM=TIMED, ERRORS) 

#include "lcd.h"
#include <string.h>
#include <stdlib.h>

long timeout=500;  // If read more than 500 characters and still no good data then break

/*------------------------------------------------------------------------------*/
/* Checksum test (Return 1 if checkum ok)				                                */
/*------------------------------------------------------------------------------*/
int checksum_ok(char *label, char *data, char checksum) 
{
	long sum=0;
	int i;
  // Add value of all characters for label
	for (i = 0; i < strlen(label); i++) sum = sum + label[i];
	// Add value of all characters for data
	for (i = 0; i < strlen(data); i++) sum = sum + data[i];
	// Add 1 space (0x20)
	sum = sum + 0x20;
	// Keep only the 6 LSB
	sum = (sum & 0x3F);
	// Add 1 space (0x20)
	sum = sum + 0x20;
	// Check if it is ok
	if (sum == checksum) return 1;
	else return 0;
}

// Here is the description of a transmission :
//
// STX (0x02 is Start of the data packet)
// LF (0x0A) Label (4 to 8 characters) SP (0x20) Data (1 to 12 charaters) SP (0x20) CRC (Character control) CR (0x0D)
// ...
// ETX (0x03 is End of the data packet)
char *readSerial(char *key)
{
  char label[11];
  char data[15];
  char crc;
  int il = 0;
  int id = 0;
  char ch = '\0';
  int field = 0;
  long i = 0;
  char to[] = "Timeout";
  char er[] = "Error";
  // Empty strings
  label[0] = '\0';
  data[0] = '\0';
  crc = '\0';
  // Loop waiting for the beginning of transmission (0x02)
  do
  {
    ch=fgetc(TIMED);
    if (RS232_ERRORS==0 && ch==0)
    {
      return(to);
    }
    i++;
  }  while ((ch != 0x02) && (i < timeout));
  // Timeout
  if (i >= timeout) return(to);
  // Read until end of transmission (0x03)
  i = 0;
  do
  {
    ch=fgetc(TIMED);
    if (RS232_ERRORS==0 && ch==0)
    {
      return(to);
    }
    // End of field line (0x0D)
    if (ch == 0x0D)
    {
      field = 0;
      il = 0;
      id = 0;
      // Test if the packet integrity is correct
      if (checksum_ok(label, data, crc))
      {
        // Test if this is the required key
        if (strcmp(label, key) == 0)
        {
          return(data);
        }
      }
      else return(er);   
    }
    // Get field 1 (Label)
    if ((field == 1) && (ch != 0x20))
    {
      label[il] = ch;
      il++;
      label[il] = '\0';   
    }
    // Get field 2 (Data)
    if ((field == 2) && (ch != 0x20))
    {
      data[id] = ch;
      id++;
      data[id] = '\0';
    }
    // Get field 3 (CRC)
    if (field == 3)
    {
      crc = ch;
    }
    // First field after (0x0A)
    if (ch == 0x0A)
    {
      field = 1;
      il = 0;
      id = 0;
    }
    // Each (0x20) is a separator for a new field
    if(ch == 0x20) field++;
    i++;
  } while ((ch != 0x03) && (i < timeout));
  // Timeout
  if (i >= timeout) return(to);
  // Normally not possible
  return(er);
}

void main() 
{
  char key[10]; 
  //All port A as output
  set_tris_a(0x00);
  //All port B as output
  set_tris_b(0x00);
  // All port D as output
  set_tris_d(0x00);
  setup_adc_ports(NO_ANALOGS);
  lcd_init();
  lcd_putc("\fUSB Ready...  V2\n");

  output_a(0x00);
  output_d(0x01);

  while (TRUE) 
  {

    strcpy(key, "HCHC");
    printf(lcd_putc, "\fHCHC= %s", readSerial(key));
    strcpy(key, "HCHP");
    printf(lcd_putc, "\nHCHP= %s", readSerial(key));
    output_toggle(PIN_D2);
    delay_ms(1);
  }
}
